/* Copyright (c) 2009 Nordic Semiconductor. All Rights Reserved.
 *
 * The information contained herein is property of Nordic Semiconductor ASA.
 * Terms and conditions of usage are described in detail in NORDIC
 * SEMICONDUCTOR STANDARD SOFTWARE LICENSE AGREEMENT.
 *
 * Licensees are granted free, non-transferable use of the information. NO
 * WARRANTY of ANY KIND is provided. This heading must NOT be removed from
 * the file.
 *
 */

/** @file
* @brief Radio Receiver example.
*
* @defgroup nrf_dev_radio_rx_example Radio Receiver example
* @{
* @ingroup nrf_examples_nrf6310
*
* @brief Example project on receiving data using the NRF_RADIO peripheral.
*
* The receiver is configured to continuously receive packets from the corresponding
* transmitter project. The first payload byte of the packet is sent to P1 which can
* be used, for example, to light up the LEDs on an nRFgo board.
*
* @image html example_board_setup_a.png "Use board setup A for this example."
*/
#include <stdint.h>
#include <stdbool.h>
#include "nrf_delay.h"
#include "radio_config.h"
#include "nrf_gpio.h"

#define MAX_PACKET_SIZE (50U)

static uint8_t volatile packet[MAX_PACKET_SIZE];  ///< Received packet buffer

/**
 * main() function
 * @return 0. int return type required by ANSI/ISO standard.
 */
int main(void)
{
    /* Start 16 MHz crystal oscillator */
    NRF_CLOCK->EVENTS_HFCLKSTARTED = 0;
    NRF_CLOCK->TASKS_HFCLKSTART = 1;

    /* Wait for the external oscillator to start up */
    while (NRF_CLOCK->EVENTS_HFCLKSTARTED == 0);

    // Set Port 1 as output
    nrf_gpio_range_cfg_output(8, 15);

    // Set radio configuration parameters
    radio_configure();

    // Enable the shortcut between getting an address match in the radio and starting the RSSI
    // This will enable the RSSI once on every received packet
    NRF_RADIO->SHORTS |= RADIO_SHORTS_ADDRESS_RSSISTART_Msk;

    nrf_gpio_port_write(NRF_GPIO_PORT_SELECT_PORT1, 0x55);

    NRF_RADIO->INTENSET = RADIO_INTENSET_READY_Msk | RADIO_INTENSET_END_Msk | RADIO_INTENSET_DISABLED_Msk;

    NVIC_EnableIRQ(RADIO_IRQn);

    nrf_gpio_port_write(NRF_GPIO_PORT_SELECT_PORT1, 0x55);

    // Set payload pointer
    NRF_RADIO->PACKETPTR = (uint32_t)packet;
    
    // Set the radio in RX once to start the receive procedure
    NRF_RADIO->TASKS_RXEN = 1;
  
    while(true)
    {
        // Nothing do be done in the main loop since everything is handled through interrupts
    }

}

void RADIO_IRQHandler()
{
    // The very first time the radio is powered up we have to trigger the START task after getting the READY event
    // (note, this could also have been handled by a shortcut)
    if(NRF_RADIO->EVENTS_READY)
    {
        NRF_RADIO->EVENTS_READY = 0;
        
        // Start listening and wait for the END event
        NRF_RADIO->TASKS_START = 1;
    }
    
    // Every time the END event is triggered we have received a packet on the radio
    // Check the CRC status on the packet, do whatever we want with it, and trigger the START task again to enable the receiver for new data
    if(NRF_RADIO->EVENTS_END)
    {
        NRF_RADIO->EVENTS_END = 0;
        
        if(NRF_RADIO->CRCSTATUS == 1)
        {
            // Convert the RSSI data so that a single LED is lit depending on the signal strength
            // On low RSSI values LED 0 will be lit, higher RSSI values will light LED 1, LED 2 and so forth..
            nrf_gpio_port_write(NRF_GPIO_PORT_SELECT_PORT1, 1 << ((NRF_RADIO->RSSISAMPLE*10/128)-2));

            //nrf_gpio_port_write(NRF_GPIO_PORT_SELECT_PORT1, packet[2]);           
        }
        
        // Start listening and wait for the END event
        NRF_RADIO->TASKS_START = 1;
    }
}

/**
 *@}
 **/
